--[[ Baggotry
     A bag addon of no specific functionality

]]--

local addoninfo, bag = ...
local lbag = Library.LibBaggotry
local filt = Library.LibEnfiltrate
bag.version = "0.16-151128-15:55:12"

bag.filters = {}

local printf = Library.printf.printf
local sprintf = Library.printf.sprintf
local printfhs = Library.printf.printfhs

function valuation(details, slot, value)
  value = value or 0
  if details.sell then
    value = value + details.sell
  end
  return value
end

bag.inventories = { "Bank", "Vault", "Inventory" }

function bag.avail(category)
  local t = Inspect.Item.List(Utility.Item.Slot[category]())
  local count = 0
  local total = 0
  for k, v in pairs(t) do
    total = total + 1
    -- ignore bag slots as such
    if v == false and k:sub(3, 4) ~= 'bg' then
      count = count + 1
    end
  end
  return count, total
end

function bag.show_avail(name, avail)
  local count = avail[1]
  local total = avail[2]
  local pct = count * 100 / total
  local color
  if pct > 90 then
    color = "44ff44"
  elseif pct > 75 then
    color = "77cc00"
  elseif pct > 50 then
    color = "aaaa00"
  elseif pct > 25 then
    color = "bb6600"
  elseif pct > 10 then
    color = "cc4400"
  elseif pct > 0 then
    color = "ff4400"
  else
    color = "cc0000"
  end
  printfhs("<font color=\"#cccccc\">  %s: <font color=\"#%s\">%d</font>/%d free.</font>", name, color, count, total)
end

function bag.slashcommand(args)
  local stack = false
  local dump = false
  local gold = false
  local merge = false
  local move = false
  local slotspecs = {}
  local filter
  local temporary = false
  local created = false
  local stack_size = nil
  local verbose = false
  if not args then
    return
  end
  if args.o then
    local bank = { bag.avail('Bank') }
    local vault = { bag.avail('Vault') }
    local inventory = { bag.avail('Inventory') }
    bag.show_avail('Bank', { bank[1] + vault[1], bank[2] + vault[2] })
    bag.show_avail('Inventory', inventory)
    return
  end
  if args.a then
    lbag.share_inventory(true)
    printf("Sharing enabled.")
    return
  end
  if args.V then
    verbose = true
  end
  if args.v then
    printf("version %s", bag.version)
    return
  end
  if args.f then
    filter = filt.Filter:load(args.f, 'Baggotry')
    if not filter then
      filter = filt.Filter:new(args.f, 'item', 'Baggotry')
      created = true
    end
    args.f = nil
  else
    filter = filt.Filter:new(nil, 'item', 'Baggotry')
    temporary = true
  end

  if args.M then
    if args.M == 'bank' then
      move = Utility.Item.Slot.Bank()
    elseif args.M == 'inventory' then
      move = Utility.Item.Slot.Inventory()
    elseif lbag.slotspec_p(args.M) then
      move = args.M
    else
      printf("Unknown slotspec '%s': should be slotspec, 'bank', or 'inventory'.", args.M)
    end
  end

  if args.d then
    if filter then
      filter:dump()
    end
    args.d = nil
  end

  if args.l then
    local ordered = {}
    local filters = filt.Filter:list('Baggotry')
    for k, v in pairs(filt.Filter:list('Baggotry')) do
      table.insert(ordered, k)
    end
    table.sort(ordered)
    for _, v in ipairs(ordered) do
      local filter = filt.Filter:load(v, 'Baggotry')
      if filter then
        filter:dump()
      else
        printf("Filter <%s>: Can't load.", v)
      end
    end
    return
  end
  if args.S then
    stack = true
    stack_size = args.S
    args.S = nil
  end
  if args.D then
    dump = true
    args.D = nil
  end
  if args.m then
    merge = true
    args.m = nil
  end
  if args.g then
    gold = true
    args.g = nil
  end

  local changed= false
  if lbag.apply_args(filter, args) then changed = true end
  if filter:apply_args(args, verbose) then changed = true end
  if changed then
    if not temporary then
      filter:save()
    end
  else
    if created then
      printf("Found no filter named '%s'.", filter.name)
      return
    end
  end

  if dump then
    filter:dump()
    return
  end

  if gold then
    local total, count

    total, count = lbag.iterate(filter, valuation)

    local silver = total % 100
    local gold = math.floor(total / 100)
    local plat = math.floor(gold / 100)
    gold = gold % 100
    if plat > 0 then
      printf("%d item(s), total value: %dp%dg%ds.", count, plat, gold, silver)
    elseif gold > 0 then
      printf("%d item(s), total value: %dg%ds.", count, gold, silver)
    elseif silver > 0 then
      printf("%d item(s), total value: %ds.", count, silver)
    else
      printf("Total value: none.")
    end
    return
  end
  if stack then
    lbag.stack(filter, stack_size, true)
  end
  if move then
    lbag.move_items(filter, move, true)
  end
  if not (stack or move) then
    local items = lbag.expand(filter)
    if merge then
      items = lbag.merge_items(items)
    end
    bag.dump(items)
  end
end

function bag.dump_item(item, prefix)
  local prettystack
  if item.stackMax and item.stackMax > 1 then
    prettystack = string.format(" [%d/%d]", item.stack or 1, item.stackMax)
  else
    prettystack = ""
  end
  local color = lbag.rarity_color(item.rarity, true) or { html = 'cccccc' }
  printfhs("<font color=\"#bbbbbb\">%s%s: <font color=\"#%s\">%s</font>%s</font>",
      prefix or "",
      lbag.slotspec_pretty(item._slotspec) or '??',
      color.html or 'cccccc',
      item.name,
      prettystack)
end

function bag.dump(item_list)
  if not next(item_list) then
    printf("No matches.")
    return
  end

  local chars = {}
  local items_by_char = {}
  local items_per_char = {}

  -- first, split by characters
  for k, v in pairs(item_list) do
    local char = v._character or 'unknown'
    items_per_char[char] = (items_per_char[char] or 0) + 1
    items_by_char[char] = items_by_char[char] or {}
    items_by_char[char][k] = v
  end

  -- make a list of characters seen so we can display in order
  local whoami = lbag.whoami()
  local other_people = false
  for k, v in pairs(items_by_char) do
    if k ~= whoami then
      chars[#chars + 1] = k
    end
  end
  if #chars > 0 then
    table.sort(chars)
    other_people = true
  end
  -- and add myself at the bottom of the list
  chars[#chars + 1] = whoami

  printf("Matching items:")
  for i = 1, #chars do
    local char = chars[i]
    if items_by_char[char] then
      local name = char:sub(1, 1):upper() .. char:sub(2, -1)
      if other_people then
        printfhs("%s:", name)
      end
      for k, v in pairs(items_by_char[char]) do
        bag.dump_item(v, other_people and "  ")
      end
    end
  end
end

Library.LibGetOpt.makeslash(filt.Filter:argstring() .. lbag.argstring() .. "ad:Df:glM:moS#vV", "Baggotry", "bag", bag.slashcommand)

